! Fortran 90 and PVM
! measure of communication performance
! Protocole between Ping and Pong to aknowlege presence of each other.
!
! Laurent Gasser (lga@sma.ch)
! Swiss Meteorological Institute
! November 21, 1996

module politeness
use PVM_Constants
use PVM_Interfaces

implicit none

public :: say_hello, hear_hello, say_goodbye, hear_goodbye

private 
  integer, parameter :: IS_READY_TAG      = 100
  integer, parameter :: HAS_COMPLETED_TAG = 101

contains
  
  subroutine say_hello(to)
    implicit none
    integer, intent(in) :: to
  ! local declarations
    integer :: buffer_ID = -1
    integer :: info      = PvmOk
  ! body say_hello
    call pvm_initsend(PVMDATADEFAULT, buffer_ID)
    if (buffer_ID < 0) then
       print *, " Pong failed to init the greeting message, cancel experiment."
       call pvm_perror("Politeness, say_hello", info)
       call exit(info)
    end if
    call pvm_pack(buffer_ID, info) ! meaningless content
    if (info /= PvmOk) then
       print *, " Pong failed to pack greeting message, cancel experiment."
       call pvm_perror("Politeness, say_hello", info)
       call exit(info)
    end if
    call pvm_send(to, IS_READY_TAG, info)
    if (info /= PvmOk) then
       print *, " Pong failed to report it is ready, cancel experiment."
       call pvm_perror("Politeness, say_hello", info)
       call exit(info)
    end if
  end subroutine say_hello

  subroutine hear_hello(from)
    implicit none
    integer, intent(in) :: from
  ! local declarations
    integer       :: buffer_ID  = -1
    integer       :: info       = PvmOK
    integer       :: not_used   = -666
    character(31) :: slave_name = "Pong"
  ! body hear_hello     
    call pvm_recv(from, IS_READY_TAG, buffer_ID)
    if (buffer_ID < 0) then
       print *, " Reception of Pong presence message failed."
       call pvm_perror("Politeness, hear_hello", info)
    end if
    call pvm_unpack(not_used, info)
    if (info /= PvmOK) then
       print *, " Unpacking of Pong presence message failed."
       call pvm_perror("Politeness, hear_hello", info)
    end if
  end subroutine hear_hello

  subroutine say_goodbye(to)
    implicit none
    integer, intent(in) :: to
  ! local declarations
    integer :: buffer_ID = -1
    integer :: info      = PvmOk
  ! body say_goodbye
    call pvm_initsend(PVMDATADEFAULT, buffer_ID)
    if (buffer_ID < 0) then
       print *, " Pong failed to init the leaving message, cancel experiment."
       call pvm_perror("Politeness, say_goodbye", info)
       call exit(info)
    end if
    call pvm_pack(buffer_ID, info) ! meaningless content
    if (info /= PvmOk) then
       print *, " Pong failed to pack leaving message, cancel experiment."
       call pvm_perror("Politeness, say_goodbye", info)
       call exit(info)
    end if
    call pvm_send(to, HAS_COMPLETED_TAG, info)
    if (info /= PvmOk) then
       print *, " Pong failed to report completion, quit silently."
       call pvm_perror("Politeness, say_goodbye", info)
    end if
  end subroutine say_goodbye

  subroutine hear_goodbye(from)
    implicit none
    integer, intent(in) :: from
  ! local declarations
    integer       :: buffer_ID  = -1
    integer       :: info       = PvmOK
    integer       :: not_used
    character(31) :: slave_name = "Pong"
  ! body hear_goodbye
    call pvm_recv(from, HAS_COMPLETED_TAG, buffer_ID)
    if (buffer_ID < 0) then
       print *, " Reception of Pong quitting message failed."
       call pvm_perror("Politeness, hear_goodbye", info)
    end if
    call pvm_unpack(not_used, info)
    if (info /= PvmOK) then
       print *, " Unpacking of Pong leaving message failed."
       call pvm_perror("Politeness, hear_goodbye", info)
    end if
  end subroutine hear_goodbye

end module politeness
